using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using OpenNETCF.Desktop.Communication;
/* Some of this code was copied from the sample program provided by OpenNETCF */

namespace LCDAdapt
{
    public partial class MainForm : BaseForm
    {
        #region members
        enum Predefines {Custom, DVI_640W_480H, DVI_640W_480H_72HZ, DVI_800W_480H, DVI_800W_600H, DVI_800W_600H_56HZ,
        DVI_1024W_768H, DVI_1280W_720H, DVI_1400W_1050H, VGA_480W_640H, TOPPOLY_480W_640H, LVDS_1024W_768H, NONE};
        private static readonly string[] PredefineNames = {"Custom", "DVI_640W_480H", "DVI_640W_480H_72HZ", 
            "DVI_800W_480H", "DVI_800W_600H", "DVI_800W_600H_56HZ","DVI_1024W_768H", "DVI_1280W_720H", 
            "DVI_1400W_1050H", "VGA_480W_640H", "TOPPOLY_480W_640H", "LVDS_1024W_768H"};
        private const int Dpll4ClockMhz = 864;
        private const string CM_X300_REGISTRY_ADDR = "Drivers\\Display\\GPE";
        private const string CM_T35_REGISTRY_ADDR  = "System\\GDI\\Drivers\\Config";
        
        private Platforms Platform;
        private ToolTip tip = new ToolTip();
        private bool bIsPredefinedKey = false;
        private bool bIsCustom = true;
        private rapi Rapi = null;

        private Double dPixelClock;
        private UInt32 uiHorizRes;
        private UInt32 uiVertRes;
        private UInt32 uiBpp;
        private UInt32 uiHorizSync;
        private UInt32 uiVertSync;
        private UInt32 uiHorizBackPorch;
        private UInt32 uiHorizFrontPorch;
        private UInt32 uiVertBackPorch;
        private UInt32 uiVertFrontPorch;
        private bool bDataEnablePolarity;
        private bool bPixelClockPolarity;
        private bool bHorizSyncPolarity;
        private bool bVertSyncPolarity;
        private bool bIsDVIEnabled;
        private bool bIsLVDSEnabled;

        private Predefines prPred;
        static private LcdSettings _lcdSetting;

        public new BaseForm.GetLcdSettings GetLcdSettings;
        public new BaseForm.SetLcdSettings SetLcdSettings;

        static public  LcdSettings lcdSetting
        {
            get
            {
                return _lcdSetting;
            }
            set
            {
                _lcdSetting = value;
            }
        }

        private SortedDictionary<String, UInt32> RegVals = null;
        #endregion

        public MainForm()
        {
            InitializeComponent();
            
           
            RegVals = new SortedDictionary<String, UInt32>();
            // Enabling all fields before connect so the export for CM-T35 will work without a device
            //this.PreDefined.Enabled = false;
            //this.PixelClockEdit.Enabled = false;
            //this.VerticalResolutionEdit.Enabled = false;
            //this.HorizontalResolutionEdit.Enabled = false;
            //this.BppEdit.Enabled = false;
            //this.VerticalSyncEdit.Enabled = false;
            //this.HorizontalSyncEdit.Enabled = false;
            //this.VerticalSyncPolarityCheck.Enabled = false;
            //this.HorizontalSyncPolarityCheck.Enabled = false;
            //this.VertBackPorchEdit.Enabled = false;
            //this.VertFrontPorchEdit.Enabled = false;
            //this.HorizBackPorchEdit.Enabled = false;
            //this.HorizFrontPorchEdit.Enabled = false;
            //this.IsDVIEnabledCheck.Enabled = false;
            //this.ClockPolarityCheck.Enabled = false;  
            //this.DataEnablePolarityCheck.Enabled = false;
            //this.DataEnableLabel.Enabled = true;
            
            try
            {
                Rapi = new rapi();
            }
            catch (Exception e)
            {
                MessageBox.Show("Failed to init, error is:\r\n" + e.Message);
                this.Close();
            }

            // Default platform is CM-T35 because it is the only one supporting export right now
            Platform = Platforms.CM_T35;
            _lcdSetting.platform = Platform;

            textUpdate = new EventHandler(TextMarshaler);
            enableUpdate = new EventHandler(EnabledMarshaler);

            this.PreDefined.Items.AddRange(new object[] {Predefines.Custom,Predefines.DVI_1024W_768H,
            Predefines.DVI_1280W_720H,Predefines.DVI_1400W_1050H,Predefines.DVI_640W_480H,
            Predefines.DVI_640W_480H_72HZ,Predefines.DVI_800W_480H,Predefines.DVI_800W_600H,
            Predefines.DVI_800W_600H_56HZ,Predefines.TOPPOLY_480W_640H,Predefines.VGA_480W_640H,Predefines.LVDS_1024W_768H});

            // wire in some ActiveSync events
            Rapi.Rapi.ActiveSync.Active += new ActiveHandler(ActiveSync_Active);
            Rapi.Rapi.ActiveSync.Disconnect += new DisconnectHandler(ActiveSync_Disconnect);
            Rapi.Rapi.ActiveSync.Listen += new ListenHandler(ActiveSync_Listen);
            Rapi.Rapi.ActiveSync.Answer += new AnswerHandler(ActiveSync_Answer);


        }

        
        private void SendDeviceBtn_Click(object sender, EventArgs e)
        {
            if (!CheckInputs())
                return;
            MsgBoxCheck.MessageBox dlg = new MsgBoxCheck.MessageBox();
            DialogResult dr =
                    dlg.Show(@"Software\Compulab\LCDAdpatCheck",
                    "DontShowAgain", DialogResult.OK,
                    "Don't show this dialog again",
                    "This action will cause the device to reboot",
                    "Notice",
                    MessageBoxButtons.OK | MessageBoxButtons.OKCancel, MessageBoxIcon.Information);
            if (dr == DialogResult.Cancel)
                return;

            RegVals.Clear();
            CalculateParameters();
            if (Platform == Platforms.CM_X300)
            {
                try
                {
                    if (Rapi.OpenRegKey(CERegistry.LocalMachine, CM_X300_REGISTRY_ADDR))
                    {
                        foreach (KeyValuePair<String, UInt32> KeyVal in RegVals)
                        {
                            try
                            {
                                Rapi.SetRegKey(KeyVal.Key, KeyVal.Value);
                            }
                            catch (Exception ex)
                            {
                                MessageBox.Show("Failed to set registry value " + ex.Message);
                                Rapi.CloseRegKey();
                                return;
                            }
                        }
                        Rapi.CloseRegKey();
                        Rapi.ResetCMX300();
                    }
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Failed to open registry key" + ex.Message);
                }
            }
            if (Platform == Platforms.CM_T35)
            {
                try
                {      
                    if (Rapi.OpenRegKey(CERegistry.LocalMachine, CM_T35_REGISTRY_ADDR))
                    {
                        if (!bIsCustom)
                        {
                            if (!bIsPredefinedKey)
                            {
                                foreach (KeyValuePair<String, UInt32> KeyVal in RegVals)
                                {
                                    try
                                    {
                                        Rapi.DeleteRegValue(KeyVal.Key);
                                    }
                                    catch (Exception ex)
                                    {
                                        MessageBox.Show("Failed to delete registry value " + ex.Message);
                                        Rapi.CloseRegKey();
                                        return;
                                    }
                                }
                            }
                            try
                            {
                                Rapi.SetRegKey("PredefinedDisplay", PredefineNames[(int)prPred]);
                            }
                            catch (Exception ex)
                            {
                                MessageBox.Show("Failed to set registry value for PredefinedDisplay key " + ex.Message);
                                Rapi.CloseRegKey();
                                return;
                            }
                            //bIsPredefinedKey = true;
                        }
                        if  (bIsCustom)
                        {
                            if (bIsPredefinedKey)
                            {
                                try
                                {
                                    Rapi.DeleteRegValue("PredefinedDisplay");
                                }
                                catch (Exception ex)
                                {
                                    MessageBox.Show("Failed to delete PredefinedDisplay registry value " + ex.Message);
                                    Rapi.CloseRegKey();
                                    return;
                                }
                            }
                            foreach (KeyValuePair<String, UInt32> KeyVal in RegVals)
                            {
                                try
                                {
                                    Rapi.SetRegKey(KeyVal.Key, KeyVal.Value);
                                }
                                catch (Exception ex)
                                {
                                    MessageBox.Show("Failed to create registry value " + ex.Message);
                                    Rapi.CloseRegKey();
                                    return;
                                }
                            }
                        }
                        Rapi.CloseRegKey();
                        Rapi.ResetCMT35();
                        DisconnectProcedure();
                    }
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Failed to open registry key" + ex.Message);
                }
            
                
            }
        }


        private bool CheckInputs()
        {
            bool OutOfRange = false;
            try
            {
                dPixelClock = ((Convert.ToDouble(this.PixelClockEdit.Text))*1000000);
                uiHorizRes = Convert.ToUInt32(this.HorizontalResolutionEdit.Text);
                uiVertRes = Convert.ToUInt32(this.VerticalResolutionEdit.Text);
                uiHorizSync = Convert.ToUInt32(this.HorizontalSyncEdit.Text);
                uiVertSync = Convert.ToUInt32(this.VerticalSyncEdit.Text);
                uiHorizBackPorch = Convert.ToUInt32(this.HorizBackPorchEdit.Text);
                uiHorizFrontPorch = Convert.ToUInt32(this.HorizFrontPorchEdit.Text);
                uiVertBackPorch = Convert.ToUInt32(this.VertBackPorchEdit.Text);
                uiVertFrontPorch = Convert.ToUInt32(this.VertFrontPorchEdit.Text);
                uiBpp = Convert.ToUInt32(this.BppEdit.Text);
                if (dPixelClock < 1000000 || dPixelClock > 104000000)
                {
                    this.PixelClockEdit.BackColor = Color.Red;
                    OutOfRange = true;
                }
                if ((uiHorizSync < 0 || uiHorizSync > 63)&&(Platform==Platforms.CM_X300))
                {
                    this.HorizontalSyncEdit.BackColor = Color.Red;
                    OutOfRange = true;
                }
                if ((uiHorizSync < 0 || uiHorizSync > 255) && (Platform == Platforms.CM_T35))
                {
                    this.HorizontalSyncEdit.BackColor = Color.Red;
                    OutOfRange = true;
                }

                if (uiVertSync< 0 || uiVertSync> 63)
                {
                    this.VerticalSyncEdit.BackColor = Color.Red;
                    OutOfRange = true;
                }
                if (OutOfRange)
                {
                    MessageBox.Show("Values in boxes marked in red are out of range");
                    return false;
                }
            }
            catch (FormatException e)
            {
                string s = e.ToString();
                MessageBox.Show("Please provide numbers in the edit boxes\r\nError is:\r\n" + s);
                return false;
            }
            catch
            {
                MessageBox.Show("Unknown error happened with Pixel Clock input");
                return false;
            }
            if (Platform == Platforms.CM_X300)
            {
                bDataEnablePolarity = this.DataEnablePolarityCheck.Checked;
                
            }
            if (Platform == Platforms.CM_T35)
            {
                bIsDVIEnabled = this.IsDVIEnabledCheck.Checked;
                bIsLVDSEnabled = this.IsLVDSEnabledCheck.Checked;
            }
            bPixelClockPolarity = this.ClockPolarityCheck.Checked;
            bHorizSyncPolarity = this.HorizontalSyncPolarityCheck.Checked;
            bVertSyncPolarity = this.VerticalSyncPolarityCheck.Checked;
            prPred = (PreDefined.SelectedItem != null) ? (Predefines)PreDefined.SelectedItem : Predefines.NONE;
            return true;
        }


        private void CalculateParameters()
        {
            if (Platform == Platforms.CM_X300)
            {
                /* Calculate all PCD's */
                /* In CM_X300 the PCDDIV is always on */
                /* All calculations are done in Mega Hertz */
                String[] FrequnciesNames = { "D0CS", "39M", "104M", "156M", "208M" };
                Double[] LocalBusFrequencies = { 60e6, 39e6, 104e6, 156e6, 208e6 };
                UInt32 PCD;

                /* First check that the PCD is not going to produce a clock too big */
                int tmpPCD = (int)((104 * 1e6 / (dPixelClock)) - 1);
                if (104 / (tmpPCD + 1) > dPixelClock)
                {
                    dPixelClock = 104 * 1e6 / (tmpPCD + 2);
                    //dPixelClock /= 1e6;
                    PixelClockEdit.Text = Convert.ToString(dPixelClock / 1e6);
                }

                for (Int32 index = 0; index < LocalBusFrequencies.Length; index++)
                {
                    PCD = (UInt32)(LocalBusFrequencies[index] / (dPixelClock) - 1);
                    RegVals.Add(FrequnciesNames[index], PCD);
                }
                _lcdSetting.plat.cm_x300.pcd.d0cs = Convert.ToInt32((LocalBusFrequencies[0] / (dPixelClock) - 1));
                _lcdSetting.plat.cm_x300.pcd._39m = Convert.ToInt32((LocalBusFrequencies[1] / (dPixelClock) - 1));
                _lcdSetting.plat.cm_x300.pcd._104m = Convert.ToInt32((LocalBusFrequencies[2] / (dPixelClock) - 1));
                _lcdSetting.plat.cm_x300.pcd._156m = Convert.ToInt32((LocalBusFrequencies[3] / (dPixelClock) - 1));
                _lcdSetting.plat.cm_x300.pcd._208m = Convert.ToInt32((LocalBusFrequencies[4] / (dPixelClock) - 1));
                /* Enter the resolution */
                RegVals.Add("VerticalSize", uiVertRes);
                _lcdSetting.plat.cm_x300.height = uiVertRes;
                RegVals.Add("HorizontalSize", uiHorizRes);
                _lcdSetting.plat.cm_x300.width = uiHorizRes;
                RegVals.Add("Bpp", uiBpp);
                _lcdSetting.plat.cm_x300.bpp = uiBpp;
                /* Calculate Specific fields */
                RegVals.Add("VSW", uiVertSync);
                _lcdSetting.plat.cm_x300.vertical.SyncPulseWidth = uiVertSync;
                RegVals.Add("HSW", uiHorizSync);
                _lcdSetting.plat.cm_x300.horizontal.SyncPulseWidth = uiHorizSync;
                RegVals.Add("VSP", Convert.ToUInt32(bVertSyncPolarity));
                _lcdSetting.plat.cm_x300.vertical.SyncPolarity = Convert.ToUInt32(bVertSyncPolarity);
                RegVals.Add("HSP", Convert.ToUInt32(bHorizSyncPolarity));
                _lcdSetting.plat.cm_x300.horizontal.SyncPolarity = Convert.ToUInt32(bHorizSyncPolarity);
                RegVals.Add("PCP", Convert.ToUInt32(bPixelClockPolarity));
                _lcdSetting.plat.cm_x300.pixelclockpolarity = Convert.ToInt32(bPixelClockPolarity);
                RegVals.Add("OEP", Convert.ToUInt32(bDataEnablePolarity));
                _lcdSetting.plat.cm_x300.dataenablepolarity = Convert.ToInt32(bDataEnablePolarity);
                RegVals.Add("EFW", uiVertBackPorch);
                _lcdSetting.plat.cm_x300.vertical.BackPorch = uiVertBackPorch;
                RegVals.Add("BFW", uiVertFrontPorch);
                _lcdSetting.plat.cm_x300.vertical.FrontPorch = uiVertFrontPorch;
                RegVals.Add("ELW", uiHorizBackPorch);
                _lcdSetting.plat.cm_x300.horizontal.BackPorch = uiHorizBackPorch;
                RegVals.Add("BLW", uiHorizFrontPorch);
                _lcdSetting.plat.cm_x300.horizontal.FrontPorch = uiHorizFrontPorch;
                /* A flag so the driver will use these values */
                RegVals.Add("Leaded", 0);
            }
            if (Platform == Platforms.CM_T35)
            {
                RegVals.Add("PixelClockFrequency", (UInt32)dPixelClock);
                _lcdSetting.plat.cm_t35.PixelClockFrequency = (uint)dPixelClock;
                RegVals.Add("Height", uiVertRes);
                _lcdSetting.plat.cm_t35.Height = uiVertRes;
                RegVals.Add("Width", uiHorizRes);
                _lcdSetting.plat.cm_t35.Width = uiHorizRes;
                RegVals.Add("HorizontalSyncPulseWidth", uiHorizSync);
                _lcdSetting.plat.cm_t35.Horizontal.SyncPulseWidth = uiHorizSync;
                RegVals.Add("VerticalSyncPulseWidth", uiVertSync);
                _lcdSetting.plat.cm_t35.Vertical.SyncPulseWidth = uiVertSync;
                RegVals.Add("VerticalBackPorch", uiVertBackPorch);
                _lcdSetting.plat.cm_t35.Vertical.BackPorch = uiVertBackPorch;
                RegVals.Add("VerticalFrontPorch", uiVertFrontPorch);
                _lcdSetting.plat.cm_t35.Vertical.FrontPorch = uiVertFrontPorch;
                RegVals.Add("HorizontalBackPorch", uiHorizBackPorch);
                _lcdSetting.plat.cm_t35.Horizontal.BackPorch = uiHorizBackPorch;
                RegVals.Add("HorizontalFrontPorch", uiHorizFrontPorch);
                _lcdSetting.plat.cm_t35.Horizontal.FrontPorch = uiHorizFrontPorch;

                RegVals.Add("HorizontalSyncPolarity", Convert.ToUInt32((!bHorizSyncPolarity)));
                _lcdSetting.plat.cm_t35.Horizontal.SyncPolarity = Convert.ToUInt32((!bHorizSyncPolarity));
                RegVals.Add("VerticalSyncPolarity", Convert.ToUInt32((!bVertSyncPolarity)));
                _lcdSetting.plat.cm_t35.Vertical.SyncPolarity = Convert.ToUInt32((!bVertSyncPolarity));
                RegVals.Add("PixelClockPolarity", Convert.ToUInt32((!bPixelClockPolarity)));
                _lcdSetting.plat.cm_t35.PixelClockPolarity = Convert.ToUInt32((!bPixelClockPolarity));
                RegVals.Add("IsDVIEnabled", Convert.ToUInt32(bIsDVIEnabled));
                _lcdSetting.plat.cm_t35.IsDVIEnabled = Convert.ToUInt32(bIsDVIEnabled);
                RegVals.Add("IsLVDSEnabled", Convert.ToUInt32(bIsLVDSEnabled));
                _lcdSetting.plat.cm_t35.IsLVDSEnabled = Convert.ToUInt32(bIsLVDSEnabled);
                _lcdSetting.IsValid = true;
            
            }
        }

        public void UpdateFieldsFromDevice()
        {
            if (Rapi.Rapi.Connected)
            {
                if (Platform == Platforms.CM_X300) 
                {
                    //this.PixelClockEdit.Enabled = true;
                    //this.VerticalResolutionEdit.Enabled = true;
                    //this.HorizontalResolutionEdit.Enabled = true;
                    //this.BppEdit.Enabled = true;
                    //this.VerticalSyncEdit.Enabled = true;
                    //this.HorizontalSyncEdit.Enabled = true;
                    //this.VerticalSyncPolarityCheck.Enabled = true;
                    //this.HorizontalSyncPolarityCheck.Enabled = true;
                    //this.VertBackPorchEdit.Enabled = true;
                    //this.VertFrontPorchEdit.Enabled = true;
                    //this.HorizBackPorchEdit.Enabled = true;
                    //this.HorizFrontPorchEdit.Enabled = true;
                    //this.ClockPolarityCheck.Enabled = true;
                    //this.DataEnablePolarityCheck.Enabled = true; 

                    UInt32 PCD = Rapi.GetRegKeyValue("104M");
                    if (PCD == 0xFFFFFFFF)
                    {
                        MessageBox.Show("Failed to read PCD registry for 104MHz");
                    }
                    else
                    {
                        Double PixelClockHerz = (104) / (PCD + 1);
                        this.PixelClockEdit.Text = Convert.ToUInt32(PixelClockHerz).ToString();
                    }
                    uiVertRes = Rapi.GetRegKeyValue("VerticalSize");
                    if (uiVertBackPorch == 0xFFFFFFFF)
                    {
                        MessageBox.Show("Failed to read VerticalSize");
                    }
                    this.VerticalResolutionEdit.Text = Convert.ToString(uiVertRes);
                    uiHorizRes = Rapi.GetRegKeyValue("HorizontalSize");
                    if (uiHorizRes == 0xFFFFFFFF)
                    {
                        MessageBox.Show("Failed to read HorizontalSize");
                    }
                    this.HorizontalResolutionEdit.Text = Convert.ToString(uiHorizRes);
                    uiBpp = Rapi.GetRegKeyValue("Bpp");
                    if (uiBpp == 0xFFFFFFFF)
                    {
                        MessageBox.Show("Failed to read uiBpp");
                    }
                    this.BppEdit.Text = Convert.ToString(uiBpp);
                    uiVertSync = Rapi.GetRegKeyValue("VSW");
                    if (uiVertSync == 0xFFFFFFFF)
                    {
                        MessageBox.Show("Failed to read VSW");
                    }
                    this.VerticalSyncEdit.Text = Convert.ToString(uiVertSync);
                    uiHorizSync = Rapi.GetRegKeyValue("HSW");
                    if (uiHorizSync == 0xFFFFFFFF)
                    {
                        MessageBox.Show("Failed to read HSW");
                    }
                    this.HorizontalSyncEdit.Text = Convert.ToString(uiHorizSync);
                    bVertSyncPolarity = Convert.ToBoolean(Rapi.GetRegKeyValue("VSP"));
                    this.VerticalSyncPolarityCheck.Checked = bVertSyncPolarity;
                    bHorizSyncPolarity = Convert.ToBoolean(Rapi.GetRegKeyValue("HSP"));
                    this.HorizontalSyncPolarityCheck.Checked = bHorizSyncPolarity;
                    bPixelClockPolarity = Convert.ToBoolean(Rapi.GetRegKeyValue("PCP"));
                    this.ClockPolarityCheck.Checked = bPixelClockPolarity;
                    bDataEnablePolarity = Convert.ToBoolean(Rapi.GetRegKeyValue("OEP"));
                    this.DataEnablePolarityCheck.Checked = bDataEnablePolarity;
                    uiVertBackPorch = Rapi.GetRegKeyValue("EFW");
                    if (uiVertBackPorch == 0xFFFFFFFF)
                    {
                        MessageBox.Show("Failed to read EFW");
                    }
                    this.VertBackPorchEdit.Text = Convert.ToString(uiVertBackPorch);
                    uiVertFrontPorch = Rapi.GetRegKeyValue("BFW");
                    if (uiVertFrontPorch == 0xFFFFFFFF)
                    {
                        MessageBox.Show("Failed to read BFW");
                    }
                    this.VertFrontPorchEdit.Text = Convert.ToString(uiVertFrontPorch);
                    uiHorizBackPorch = Rapi.GetRegKeyValue("ELW");
                    if (uiHorizBackPorch == 0xFFFFFFFF)
                    {
                        MessageBox.Show("Failed to read ELW");
                    }
                    this.HorizBackPorchEdit.Text = Convert.ToString(uiHorizBackPorch);
                    uiHorizFrontPorch = Rapi.GetRegKeyValue("BLW");
                    if (uiHorizFrontPorch == 0xFFFFFFFF)
                    {
                        MessageBox.Show("Failed to read BLW");
                    }
                    this.HorizFrontPorchEdit.Text = Convert.ToString(uiHorizFrontPorch);
                
                }
                else if (Platform == Platforms.CM_T35)
                {
                    // Enabling all fields before connect so the export for CM-T35 will work without a device
                    //this.PreDefined.Enabled = true;
                    //this.PixelClockEdit.Enabled = true;
                    //this.VerticalResolutionEdit.Enabled = true;
                    //this.HorizontalResolutionEdit.Enabled = true;
                    //this.BppEdit.Enabled = true;
                    //this.VerticalSyncEdit.Enabled = true;
                    //this.HorizontalSyncEdit.Enabled = true;
                    //this.VerticalSyncPolarityCheck.Enabled = true;
                    //this.HorizontalSyncPolarityCheck.Enabled = true;
                    //this.VertBackPorchEdit.Enabled = true;
                    //this.VertFrontPorchEdit.Enabled = true;
                    //this.HorizBackPorchEdit.Enabled = true;
                    //this.HorizFrontPorchEdit.Enabled = true;
                    //this.IsDVIEnabledCheck.Enabled = true;
                    //this.ClockPolarityCheck.Enabled = true;
                    //this.DataEnableLabel.Enabled = false;

                    string sData;
                    UInt32 uiResult = Rapi.GetRegKeyValue("PredefinedDisplay", out sData);
                    if (uiResult == 0)
                    {
                        bIsPredefinedKey = true;
                        switch (sData)
                        {
                            case ("DVI_1024W_768H"):
                                FillPredefindFields(Predefines.DVI_1024W_768H);
                                break;
                            case ("DVI_640W_480H"):
                                FillPredefindFields(Predefines.DVI_640W_480H);
                                break;
                            case ("DVI_640W_480H_72HZ"):
                                FillPredefindFields(Predefines.DVI_640W_480H_72HZ);
                                break;
                            case ("DVI_800W_480H"):
                                FillPredefindFields(Predefines.DVI_800W_480H);
                                break;
                            case ("DVI_800W_600H_56HZ"):
                                FillPredefindFields(Predefines.DVI_800W_600H_56HZ);
                                break;
                            case ("DVI_800W_600H"):
                                FillPredefindFields(Predefines.DVI_800W_600H);
                                break;
                            case ("DVI_1280W_720H"):
                                FillPredefindFields(Predefines.DVI_1280W_720H);
                                break;
                            case ("DVI_1400W_1050H"):
                                FillPredefindFields(Predefines.DVI_1400W_1050H);
                                break;
                            case ("TOPPOLY_480W_640H"):
                                FillPredefindFields(Predefines.TOPPOLY_480W_640H);
                                break;
                            case ("VGA_480W_640H"):
                                FillPredefindFields(Predefines.VGA_480W_640H);
                                break;
                            case ("LVDS_1024W_768H"):
                                FillPredefindFields(Predefines.LVDS_1024W_768H);
                                break;
                            default:
                                MessageBox.Show("There is no data for PredefinedDisplay = " + sData +". Please use custom settings.");
                                break;
                        }
                        
                    }
                    else if (uiResult == 0xFFFFFFFF)
                    {

                        bIsPredefinedKey = false;
                        UInt32 temp = 0;
                        bool bTemp;
                        temp = Rapi.GetRegKeyValue("PixelClockFrequency");
                        if (temp == 0xFFFFFFFF)
                        {
                            MessageBox.Show("Failed to read PixelClockFrequency registry value");
                        }
                        this.PixelClockEdit.Text = Convert.ToString(temp/1000000);
                        temp = Rapi.GetRegKeyValue("Height");
                        if (temp == 0xFFFFFFFF)
                        {
                            MessageBox.Show("Failed to read Height registry value");
                        }
                        this.HorizontalResolutionEdit.Text = Convert.ToString(temp);
                        temp = Rapi.GetRegKeyValue("Width");
                        if (temp == 0xFFFFFFFF)
                        {
                            MessageBox.Show("Failed to read Width registry value");
                        }
                        this.VerticalResolutionEdit.Text = Convert.ToString(temp);
                        temp = Rapi.GetRegKeyValue("HorizontalSyncPulseWidth");
                        if (temp == 0xFFFFFFFF)
                        {
                            MessageBox.Show("Failed to read HorizontalSyncPulseWidth registry value");
                        }
                        this.HorizontalSyncEdit.Text = Convert.ToString(temp);
                        temp = Rapi.GetRegKeyValue("VerticalSyncPulseWidth");
                        if (temp == 0xFFFFFFFF)
                        {
                            MessageBox.Show("Failed to read VerticalSyncPulseWidth registry value");
                        }
                        this.VerticalSyncEdit.Text = Convert.ToString(temp);
                        temp = Rapi.GetRegKeyValue("VerticalBackPorch");
                        if (temp == 0xFFFFFFFF)
                        {
                            MessageBox.Show("Failed to read VerticalBackPorch registry value");
                        }
                        this.VertBackPorchEdit.Text = Convert.ToString(temp);
                        temp = Rapi.GetRegKeyValue("VerticalFrontPorch");
                        if (temp == 0xFFFFFFFF)
                        {
                            MessageBox.Show("Failed to read VerticalFrontPorch registry value");
                        }
                        this.VertFrontPorchEdit.Text = Convert.ToString(temp);
                        temp = Rapi.GetRegKeyValue("HorizontalBackPorch");
                        if (temp == 0xFFFFFFFF)
                        {
                            MessageBox.Show("Failed to read HorizontalBackPorch registry value");
                        }
                        this.HorizBackPorchEdit.Text = Convert.ToString(temp);
                        temp = Rapi.GetRegKeyValue("HorizontalFrontPorch");
                        if (temp == 0xFFFFFFFF)
                        {
                            MessageBox.Show("Failed to read HorizontalFrontPorch registry value");
                        }
                        this.HorizFrontPorchEdit.Text = Convert.ToString(temp);
                        bTemp = Convert.ToBoolean(Rapi.GetRegKeyValue("HorizontalSyncPolarity"));
                        this.HorizontalSyncPolarityCheck.Checked = !bTemp;
                        bTemp = Convert.ToBoolean(Rapi.GetRegKeyValue("VerticalSyncPolarity"));
                        this.VerticalSyncPolarityCheck.Checked = !bTemp;
                        bTemp = Convert.ToBoolean(Rapi.GetRegKeyValue("PixelClockPolarity"));
                        this.ClockPolarityCheck.Checked = !bTemp;
                        bTemp = Convert.ToBoolean(Rapi.GetRegKeyValue("IsLVDSEnabled"));
                        this.IsLVDSEnabledCheck.Checked = bTemp;
                        bTemp = Convert.ToBoolean(Rapi.GetRegKeyValue("IsDVIEnabled"));
                        this.IsDVIEnabledCheck.Checked = bTemp;
                        if (bTemp)
                            this.BppEdit.Text = "24";
                        else
                            this.BppEdit.Text = "16";
                        this.PreDefined.SelectedItem = Predefines.Custom;
                    }
                
                }
                
                Rapi.CloseRegKey();
            }
        }
        private void FillPredefindFields(Predefines pr)
        {
            bIsCustom = true;
            switch (pr)
            {
                case (Predefines.Custom):
                    this.PreDefined.SelectedItem = Predefines.Custom;
                    break;
                case (Predefines.DVI_1024W_768H):
                    {
                        this.PreDefined.SelectedItem = Predefines.DVI_1024W_768H;
                        this.PixelClockEdit.Text = Convert.ToString(Dpll4ClockMhz / 2 / 7);  //864/2/7
                        this.VerticalResolutionEdit.Text = "768";
                        this.HorizontalResolutionEdit.Text = "1024";
                        this.BppEdit.Text = "24"; // 16 TOPOLY
                        this.VerticalSyncEdit.Text = "5";
                        this.HorizontalSyncEdit.Text = "63"; // should be 135, out of range for DISPC
                        this.VerticalSyncPolarityCheck.Checked = false;
                        this.HorizontalSyncPolarityCheck.Checked = false;
                        this.VertBackPorchEdit.Text = "29";
                        this.VertFrontPorchEdit.Text = "3";
                        this.HorizBackPorchEdit.Text = "231";
                        this.HorizFrontPorchEdit.Text = "23";
                        this.IsDVIEnabledCheck.Checked = true;
                        this.ClockPolarityCheck.Checked = true;
                        this.IsLVDSEnabledCheck.Checked = false;
                        break;
                    }
                case (Predefines.LVDS_1024W_768H):
                    {
                        this.PreDefined.SelectedItem = Predefines.LVDS_1024W_768H;
                        this.PixelClockEdit.Text = Convert.ToString(Dpll4ClockMhz / 2 / 7);  //864/2/7
                        this.VerticalResolutionEdit.Text = "768";
                        this.HorizontalResolutionEdit.Text = "1024";
                        this.BppEdit.Text = "24"; // 16 TOPOLY
                        this.VerticalSyncEdit.Text = "5";
                        this.HorizontalSyncEdit.Text = "63"; // should be 135, out of range for DISPC
                        this.VerticalSyncPolarityCheck.Checked = false;
                        this.HorizontalSyncPolarityCheck.Checked = false;
                        this.VertBackPorchEdit.Text = "29";
                        this.VertFrontPorchEdit.Text = "3";
                        this.HorizBackPorchEdit.Text = "231";
                        this.HorizFrontPorchEdit.Text = "23";
                        this.IsDVIEnabledCheck.Checked = true;
                        this.ClockPolarityCheck.Checked = true;
                        this.IsLVDSEnabledCheck.Checked = true;
                        break;
                    }
                case (Predefines.DVI_1280W_720H):
                    {
                        this.PreDefined.SelectedItem = Predefines.DVI_1280W_720H;
                        this.PixelClockEdit.Text = Convert.ToString(Dpll4ClockMhz / 2 / 6);
                        this.VerticalResolutionEdit.Text = "720";
                        this.HorizontalResolutionEdit.Text = "1280";
                        this.BppEdit.Text = "24"; // 16 TOPOLY
                        this.VerticalSyncEdit.Text = "4";
                        this.HorizontalSyncEdit.Text = "63";  // should be 127, out of range for DISPC
                        this.VerticalSyncPolarityCheck.Checked = false;
                        this.HorizontalSyncPolarityCheck.Checked = false;
                        this.VertBackPorchEdit.Text = "20";
                        this.VertFrontPorchEdit.Text = "3";
                        this.HorizBackPorchEdit.Text = "255";
                        this.HorizFrontPorchEdit.Text = "63";
                        this.IsDVIEnabledCheck.Checked = true;
                        this.ClockPolarityCheck.Checked = true;
                        this.IsLVDSEnabledCheck.Checked = false;
                        break;
                    }
                case (Predefines.DVI_640W_480H):
                    {
                        this.PreDefined.SelectedItem = Predefines.DVI_640W_480H;
                        this.PixelClockEdit.Text = Convert.ToString(Dpll4ClockMhz / 2 / 17);
                        this.VerticalResolutionEdit.Text = "480";
                        this.HorizontalResolutionEdit.Text = "640";
                        this.BppEdit.Text = "24"; // 16 TOPOLY
                        this.VerticalSyncEdit.Text = "1";
                        this.HorizontalSyncEdit.Text = "63";  // should be 95, out of range for DISPC
                        this.VerticalSyncPolarityCheck.Checked = true;
                        this.HorizontalSyncPolarityCheck.Checked = true;
                        this.VertBackPorchEdit.Text = "33";
                        this.VertFrontPorchEdit.Text = "10";
                        this.HorizBackPorchEdit.Text = "79";
                        this.HorizFrontPorchEdit.Text = "15";
                        this.IsDVIEnabledCheck.Checked = true;
                        this.ClockPolarityCheck.Checked = true;
                        this.IsLVDSEnabledCheck.Checked = false;
                        break;
                    }
                case (Predefines.DVI_640W_480H_72HZ):
                    {
                        this.PreDefined.SelectedItem = Predefines.DVI_640W_480H_72HZ;
                        this.PixelClockEdit.Text = Convert.ToString(Dpll4ClockMhz / 4 / 7);
                        this.VerticalResolutionEdit.Text = "480";
                        this.HorizontalResolutionEdit.Text = "640";
                        this.BppEdit.Text = "24"; // 16 TOPOLY
                        this.VerticalSyncEdit.Text = "2";
                        this.HorizontalSyncEdit.Text = "39";
                        this.VerticalSyncPolarityCheck.Checked = true;
                        this.HorizontalSyncPolarityCheck.Checked = true;
                        this.VertBackPorchEdit.Text = "28";
                        this.VertFrontPorchEdit.Text = "9";
                        this.HorizBackPorchEdit.Text = "127";
                        this.HorizFrontPorchEdit.Text = "23";
                        this.IsDVIEnabledCheck.Checked = true;
                        this.ClockPolarityCheck.Checked = true;
                        this.IsLVDSEnabledCheck.Checked = false;
                        break;
                    }
                case (Predefines.DVI_800W_480H):
                    {
                        this.PreDefined.SelectedItem = Predefines.DVI_800W_480H;
                        this.PixelClockEdit.Text = Convert.ToString(Dpll4ClockMhz / 2 / 11);
                        this.VerticalResolutionEdit.Text = "480";
                        this.HorizontalResolutionEdit.Text = "800";
                        this.BppEdit.Text = "24"; // 16 TOPOLY
                        this.VerticalSyncEdit.Text = "2";
                        this.HorizontalSyncEdit.Text = "55";
                        this.VerticalSyncPolarityCheck.Checked = false;
                        this.HorizontalSyncPolarityCheck.Checked = false;
                        this.VertBackPorchEdit.Text = "68";
                        this.VertFrontPorchEdit.Text = "70";
                        this.HorizBackPorchEdit.Text = "135";
                        this.HorizFrontPorchEdit.Text = "31";
                        this.IsDVIEnabledCheck.Checked = true;
                        this.ClockPolarityCheck.Checked = true;
                        this.IsLVDSEnabledCheck.Checked = false;
                        break;
                    }
                case (Predefines.DVI_800W_600H):
                    {
                        this.PreDefined.SelectedItem = Predefines.DVI_800W_600H;
                        this.PixelClockEdit.Text = Convert.ToString(Dpll4ClockMhz / 2 / 11);
                        this.VerticalResolutionEdit.Text = "600";
                        this.HorizontalResolutionEdit.Text = "800";
                        this.BppEdit.Text = "24"; // 16 TOPOLY
                        this.VerticalSyncEdit.Text = "3";
                        this.HorizontalSyncEdit.Text = "63";
                        this.VerticalSyncPolarityCheck.Checked = false;
                        this.HorizontalSyncPolarityCheck.Checked = false;
                        this.VertBackPorchEdit.Text = "23";
                        this.VertFrontPorchEdit.Text = "1";
                        this.HorizBackPorchEdit.Text = "151";
                        this.HorizFrontPorchEdit.Text = "39";
                        this.IsDVIEnabledCheck.Checked = true;
                        this.ClockPolarityCheck.Checked = true;
                        this.IsLVDSEnabledCheck.Checked = false;
                        break;
                    }
                case (Predefines.DVI_800W_600H_56HZ):
                    {
                        this.PreDefined.SelectedItem = Predefines.DVI_800W_600H_56HZ;
                        this.PixelClockEdit.Text = Convert.ToString(Dpll4ClockMhz / 4 / 6);
                        this.VerticalResolutionEdit.Text = "600";
                        this.HorizontalResolutionEdit.Text = "800";
                        this.BppEdit.Text = "24"; // 16 TOPOLY
                        this.VerticalSyncEdit.Text = "1";
                        this.HorizontalSyncEdit.Text = "63"; // should be 71, out of range for DISPC
                        this.VerticalSyncPolarityCheck.Checked = false;
                        this.HorizontalSyncPolarityCheck.Checked = false;
                        this.VertBackPorchEdit.Text = "22";
                        this.VertFrontPorchEdit.Text = "1";
                        this.HorizBackPorchEdit.Text = "135";
                        this.HorizFrontPorchEdit.Text = "23";
                        this.IsDVIEnabledCheck.Checked = true;
                        this.ClockPolarityCheck.Checked = true;
                        this.IsLVDSEnabledCheck.Checked = false;
                        break;
                    }
                case (Predefines.DVI_1400W_1050H):
                    {
                        this.PreDefined.SelectedItem = Predefines.DVI_1400W_1050H;
                        this.PixelClockEdit.Text = Convert.ToString(Dpll4ClockMhz / 2 / 5);
                        this.VerticalResolutionEdit.Text = "1050";
                        this.HorizontalResolutionEdit.Text = "1400";
                        this.BppEdit.Text = "24"; // 16 TOPOLY
                        this.VerticalSyncEdit.Text = "4";
                        this.HorizontalSyncEdit.Text = "63"; // should be 127, out of range for DISPC
                        this.VerticalSyncPolarityCheck.Checked = false;
                        this.HorizontalSyncPolarityCheck.Checked = false;
                        this.VertBackPorchEdit.Text = "20";
                        this.VertFrontPorchEdit.Text = "3";
                        this.HorizBackPorchEdit.Text = "255";
                        this.HorizFrontPorchEdit.Text = "63";
                        this.IsDVIEnabledCheck.Checked = true;
                        this.ClockPolarityCheck.Checked = true;
                        this.IsLVDSEnabledCheck.Checked = false;
                        break;
                    }
                case (Predefines.VGA_480W_640H):
                    {
                        this.PreDefined.SelectedItem = Predefines.VGA_480W_640H;
                        this.PixelClockEdit.Text = Convert.ToString(Dpll4ClockMhz / 2 / 6);
                        this.VerticalResolutionEdit.Text = "480";
                        this.HorizontalResolutionEdit.Text = "640";
                        this.BppEdit.Text = "24"; // 16 TOPOLY
                        this.VerticalSyncEdit.Text = "0";
                        this.HorizontalSyncEdit.Text = "1";
                        this.VerticalSyncPolarityCheck.Checked = true;
                        this.HorizontalSyncPolarityCheck.Checked = true;
                        this.VertBackPorchEdit.Text = "2";
                        this.VertFrontPorchEdit.Text = "5";
                        this.HorizBackPorchEdit.Text = "77";
                        this.HorizFrontPorchEdit.Text = "87";
                        this.IsDVIEnabledCheck.Checked = false;
                        this.ClockPolarityCheck.Checked = true;
                        this.IsLVDSEnabledCheck.Checked = false;
                        break;
                    }
                case (Predefines.TOPPOLY_480W_640H):
                    {
                        this.PreDefined.SelectedItem = Predefines.TOPPOLY_480W_640H;
                        this.PixelClockEdit.Text = Convert.ToString(Dpll4ClockMhz / 7 / 5);
                        this.VerticalResolutionEdit.Text = "480";
                        this.HorizontalResolutionEdit.Text = "640";
                        this.BppEdit.Text = "16";
                        this.VerticalSyncEdit.Text = "1";
                        this.HorizontalSyncEdit.Text = "32";
                        this.VerticalSyncPolarityCheck.Checked = true;
                        this.HorizontalSyncPolarityCheck.Checked = true;
                        this.VertBackPorchEdit.Text = "3";
                        this.VertFrontPorchEdit.Text = "4";
                        this.HorizBackPorchEdit.Text = "23";
                        this.HorizFrontPorchEdit.Text = "8";
                        this.IsDVIEnabledCheck.Checked = false;
                        this.ClockPolarityCheck.Checked = true;
                        this.IsLVDSEnabledCheck.Checked = false;
                        break;
                    }
                default:
                    MessageBox.Show("FillPredefindFields Error!");
                    break;
            }
            bIsCustom = false;
        
        }
        private void SetToolTips()
        {
            tip.AutoPopDelay = 5000;
            tip.InitialDelay = 1000;
            tip.ReshowDelay = 500;
            // Force the ToolTip text to be displayed whether or not the form is active.
            tip.ShowAlways = true;

            // Set up the ToolTip text for the Button and Checkbox.
            string text;
            text = "Name of the connected module";
            tip.SetToolTip(this.deviceName, text);
            text = "Connection status of the application";
            tip.SetToolTip(this.connectStatus, text);
            if (Platform == Platforms.CM_X300)
            {
                text = "This is calculated to the nearest possible value\r\n";
                text += "The actual formula is: PixelClock = 104MHz / (X+1)\r\n";
                text += "Where X is an integer.\n\r";
                text += "The result is lowered to the nearest supported clock value.";
                tip.SetToolTip(this.pClkText, text);
                tip.SetToolTip(this.PixelClockEdit, text);
                text = "When this is checked PCP equals 1.\r\n" +
                    "When checked, it means the clock is active on falling edge.";
                tip.SetToolTip(this.ClockPolarityCheck, text);
                tip.SetToolTip(this.pClockPolarityText, text);
                text = "When checked OEP is 1.\r\n" +
                    "When checked the data enable polarity is low.";
                tip.SetToolTip(this.DataEnablePolarityCheck, text);
                text = "Sets the number of horizontal pixels.";
                tip.SetToolTip(this.HorizontalResolutionEdit, text);
                tip.SetToolTip(this.hResText, text);
                text = "Sets the number of vertical pixels.";
                tip.SetToolTip(this.VerticalResolutionEdit, text);
                tip.SetToolTip(this.VResText, text);
                text = "Sets HSW field in the registry.\r\n" +
                    "specifies the pulse width (minus 1) of \r\n" +
                    "the line clock in passive mode or the horizontal\r\n" +
                    "synchronization pulse in active mode.";
                tip.SetToolTip(this.HorizontalSyncEdit, text);
                tip.SetToolTip(this.HorizSyncText, text);
                text = "When this is checked HSP equals 1.\r\n" +
                    "When checked, it means the clock is active on falling edge.";
                tip.SetToolTip(this.HorizontalSyncPolarityCheck, text);
                tip.SetToolTip(this.HorizSyncText, text);
                text = "When this is checked VSP equals 1.\r\n" +
                    "When checked, it means the clock is active on falling edge.";
                tip.SetToolTip(this.VerticalSyncPolarityCheck, text);
                tip.SetToolTip(this.VertSyncText, text);
            }
            if (Platform == Platforms.CM_T35) 
            {
                text = "Pixel clock frequency\r\n";
                text += "Must be within range: 1-104";
                tip.SetToolTip(this.pClkText, text);
                tip.SetToolTip(this.PixelClockEdit, text);
                text = "If checked, clock is falling edge triggered.\r\n";
                tip.SetToolTip(this.ClockPolarityCheck, text);
                tip.SetToolTip(this.pClockPolarityText, text);
                text = "Number of distinct pixels in horizontal dimension.";
                tip.SetToolTip(this.HorizontalResolutionEdit, text);
                tip.SetToolTip(this.hResText, text);
                text = "Number of distinct pixels in vertical dimension.";
                tip.SetToolTip(this.VerticalResolutionEdit, text);
                tip.SetToolTip(this.VResText, text);
                text = "Number of bits used to represent the color of a single pixel.\r\n" +
                    "24 if DVI is enabled, 16 otherwise.\r\n";
                tip.SetToolTip(this.BppLabel, text);
                tip.SetToolTip(this.BppEdit, text);
                text = "If checked, DVI interface is enabled.";
                tip.SetToolTip(this.DviLabel, text);
                tip.SetToolTip(this.IsDVIEnabledCheck, text);
                text = "If checked, LVDS interface is enabled.";
                tip.SetToolTip(this.LvdsLabel, text);
                tip.SetToolTip(this.IsLVDSEnabledCheck, text);
                text = "Number of predefined display settings sets.";
                tip.SetToolTip(this.PreDefined, text);
                text = "Horizontal synchronizing pulse width.\r\n" +
                       "Must be within range: 0-255.";
                tip.SetToolTip(this.HorizontalSyncEdit, text);
                tip.SetToolTip(this.HorizSyncText, text);
                text = "Vertical synchronizing pulse width.\r\n" +
                       "Must be within range: 0-63.";
                tip.SetToolTip(this.VerticalSyncEdit, text);
                tip.SetToolTip(this.VertSyncText, text);
                text = "Horizontal back-porch.";
                tip.SetToolTip(this.HorizBackPorchEdit, text);
                tip.SetToolTip(this.HBackPorchText, text);
                text = "Horizontal front-porch.";
                tip.SetToolTip(this.HorizFrontPorchEdit, text);
                tip.SetToolTip(this.HFrontPorchText, text);
                text = "Vertical back-porch.";
                tip.SetToolTip(this.VertBackPorchEdit, text);
                tip.SetToolTip(this.VBackPorchText, text);
                text = "Vertical front-porch.";
                tip.SetToolTip(this.VertFrontPorchEdit, text);
                tip.SetToolTip(this.VFrontPorchText, text);
                text = "If checked, horizontal synchronizing pulse is falling edge triggered.\r\n";
                tip.SetToolTip(this.hSyncPolLabel, text);
                tip.SetToolTip(this.HorizontalSyncPolarityCheck, text);
                text = "If checked, vertical synchronizing pulse is falling edge triggered.\r\n";
                tip.SetToolTip(this.vSyncPolLabel, text);
                tip.SetToolTip(this.VerticalSyncPolarityCheck, text);
                
              
            }
        }
        
        private void UpdateSettingsForExport()
        {
            if (CheckInputs())
            {
                RegVals.Clear();
                CalculateParameters();
                SetLcdSettings(_lcdSetting);
            }
        }

        #region marshaling and text updating
        internal class TextArgs : EventArgs
        {
            public TextArgs(Control target, string text)
            {
                Target = target;
                Text = text;
            }

            public Control Target;
            public string Text;
        }

        internal class EnableArgs : EventArgs
        {
            public EnableArgs(Control target, bool enabled)
            {
                Target = target;
                Enabled = enabled;
            }

            public Control Target;
            public bool Enabled;
        }

        private EventHandler textUpdate;
        private EventHandler enableUpdate;

        private void TextMarshaler(object o, EventArgs args)
        {
            TextArgs arg = (TextArgs)args;
            arg.Target.Text = arg.Text;
        }

        private void EnabledMarshaler(object o, EventArgs args)
        {
            EnableArgs arg = (EnableArgs)args;
            arg.Target.Enabled = arg.Enabled;
        }
        #endregion

        #region ActiveSync events and delegates
        private void ActiveSync_Active()
        {
            this.Invoke(textUpdate, new object[] { this, new TextArgs(status, "Connected") });
            this.SendDeviceBtn.Enabled = true;
            this.Update();
        }

        private void ActiveSync_Disconnect()
        {
            this.Invoke(textUpdate, new object[] { this, new TextArgs(status, "Disconnected") });
            this.SendDeviceBtn.Enabled = false;
            this.Update();
        }

        private void ActiveSync_Listen()
        {
            this.Invoke(textUpdate, new object[] { this, new TextArgs(status, "Not Connected") });
            this.SendDeviceBtn.Enabled = false;
            this.Update();
        }

        private void ActiveSync_Answer()
        {
            this.Invoke(textUpdate, new object[] { this, new TextArgs(status, "Connecting...") });
            this.SendDeviceBtn.Enabled = false;
            this.Update();
        }

        private void m_rapi_RAPIConnected()
        {
            if (true == Rapi.OpenRegKey(CERegistry.LocalMachine, CM_X300_REGISTRY_ADDR))
            {
                Platform = Platforms.CM_X300;
                PreDefined.Enabled = false;
                IsDVIEnabledCheck.Enabled = false;
                IsLVDSEnabledCheck.Enabled = false;
                this.Invoke(textUpdate, new object[] { this, new TextArgs(deviceName, "PXA") });

            }
            else if (true == Rapi.OpenRegKey(CERegistry.LocalMachine, CM_T35_REGISTRY_ADDR))
            {
                Platform = Platforms.CM_T35;
                this.Invoke(textUpdate, new object[] { this, new TextArgs(deviceName, "CM-T35") });
            }
            else
            {
                MessageBox.Show("Connected device is not supported.");
                Rapi.Rapi.Disconnect();
            }
            _lcdSetting.platform = Platform;
            this.Invoke(textUpdate, new object[] { this, new TextArgs(connectStatus, "Connected") });
            SetToolTips();
            this.UpdateFieldsFromDevice();

            //Update lcdSettings struct
            UpdateSettingsForExport();

            this.Invoke(enableUpdate, new object[] { this, new EnableArgs(SendDeviceBtn, true) });
            this.Invoke(enableUpdate, new object[] { this, new EnableArgs(btn_disconnect, true) });
            this.Invoke(enableUpdate, new object[] { this, new EnableArgs(btn_connect, false) });
        }

        private void m_rapi_RAPIDisconnected()
        {
            this.Invoke(textUpdate, new object[] { this, new TextArgs(connectStatus, "Not Connected") });
            this.Invoke(enableUpdate, new object[] { this, new EnableArgs(SendDeviceBtn, false) });
            this.Invoke(enableUpdate, new object[] { this, new EnableArgs(btn_disconnect, false) });
            this.Invoke(enableUpdate, new object[] { this, new EnableArgs(btn_connect, true) });
        }
        #endregion
        
        #region button events
        private void button2_Click(object sender, EventArgs e)
        {
            DisconnectProcedure();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            Rapi.Rapi.RAPIConnected += new RAPIConnectedHandler(m_rapi_RAPIConnected);
            Rapi.Rapi.RAPIDisconnected += new RAPIConnectedHandler(m_rapi_RAPIDisconnected);
            Rapi.Rapi.Connect(true);
        }
        #endregion

        #region fields update events
        //Each field change causes the global LcdSettings struct to update

        private void DisconnectProcedure()
        {
            Rapi.Rapi.Disconnect();
            connectStatus.Text = "Disconnected";
            // Enabling all fields before connect so the export for CM-T35 will work without a device
            //this.PreDefined.Enabled = false;
            //this.PixelClockEdit.Enabled = false;
            //this.VerticalResolutionEdit.Enabled = false;
            //this.HorizontalResolutionEdit.Enabled = false;
            //this.BppEdit.Enabled = false;
            //this.VerticalSyncEdit.Enabled = false;
            //this.HorizontalSyncEdit.Enabled = false;
            //this.VerticalSyncPolarityCheck.Enabled = false;
            //this.HorizontalSyncPolarityCheck.Enabled = false;
            //this.VertBackPorchEdit.Enabled = false;
            //this.VertFrontPorchEdit.Enabled = false;
            //this.HorizBackPorchEdit.Enabled = false;
            //this.HorizFrontPorchEdit.Enabled = false;
            //this.IsDVIEnabledCheck.Enabled = false;
            //this.ClockPolarityCheck.Enabled = false;
            //this.DataEnablePolarityCheck.Enabled = false;
            //this.DataEnableLabel.Enabled = true;
            SendDeviceBtn.Enabled = false;
            btn_connect.Enabled = true;
            btn_disconnect.Enabled = false;
        }

        private void HorizontalSyncEdit_TextChanged(object sender, EventArgs e)
        {
            this.HorizontalSyncEdit.BackColor = Color.White;
            if ((Platform == Platforms.CM_T35) && !bIsCustom)
            {
                this.PreDefined.SelectedItem = Predefines.Custom;
                bIsCustom = true;
            }
            UpdateSettingsForExport();
        }

        private void VerticalSyncEdit_TextChanged(object sender, EventArgs e)
        {
            this.VerticalSyncEdit.BackColor = Color.White;
            if ((Platform == Platforms.CM_T35) && !bIsCustom)
            {
                this.PreDefined.SelectedItem = Predefines.Custom;
                bIsCustom = true;
            }
            UpdateSettingsForExport();
        }

        private void PixelClockEdit_TextChanged(object sender, EventArgs e)
        {
            this.PixelClockEdit.BackColor = Color.White;
            if ((Platform == Platforms.CM_T35) && !bIsCustom)
            {
                this.PreDefined.SelectedItem = Predefines.Custom;
                bIsCustom = true;
            }
            UpdateSettingsForExport();
        }

        private void PreDefined_SelectedIndexChanged(object sender, EventArgs e)
        {
            FillPredefindFields((Predefines)this.PreDefined.SelectedItem);
            UpdateSettingsForExport();
        }

        private void HorizontalResolutionEdit_TextChanged(object sender, EventArgs e)
        {
            if ((Platform == Platforms.CM_T35) && !bIsCustom)
            {
                this.PreDefined.SelectedItem = Predefines.Custom;
                bIsCustom = true;
            }
            UpdateSettingsForExport();
        }

        private void VerticalResolutionEdit_TextChanged(object sender, EventArgs e)
        {
            if ((Platform == Platforms.CM_T35) && !bIsCustom)
            {
                this.PreDefined.SelectedItem = Predefines.Custom;
                bIsCustom = true;
            }
            UpdateSettingsForExport();
        }

        private void BppEdit_TextChanged(object sender, EventArgs e)
        {
            if ((Platform == Platforms.CM_T35) && !bIsCustom)
            {
                this.PreDefined.SelectedItem = Predefines.Custom;
                bIsCustom = true;
            }
            UpdateSettingsForExport();
        }

        private void IsDVIEnabledCheck_CheckedChanged(object sender, EventArgs e)
        {
            if ((Platform == Platforms.CM_T35) && !bIsCustom)
            {
                this.PreDefined.SelectedItem = Predefines.Custom;
                bIsCustom = true;
            }
            UpdateSettingsForExport();
        }
        private void IsLVDSEnabledCheck_CheckedChanged(object sender, EventArgs e)
        {
            if ((Platform == Platforms.CM_T35) && !bIsCustom)
            {
                this.PreDefined.SelectedItem = Predefines.Custom;
                bIsCustom = true;
            }
            UpdateSettingsForExport();
        }

        private void HorizBackPorchEdit_TextChanged(object sender, EventArgs e)
        {
            if ((Platform == Platforms.CM_T35) && !bIsCustom)
            {
                this.PreDefined.SelectedItem = Predefines.Custom;
                bIsCustom = true;
            }
            UpdateSettingsForExport();
        }

        private void HorizFrontPorchEdit_TextChanged(object sender, EventArgs e)
        {
            if ((Platform == Platforms.CM_T35) && !bIsCustom)
            {
                this.PreDefined.SelectedItem = Predefines.Custom;
                bIsCustom = true;
            }
            UpdateSettingsForExport();
        }

        private void HorizontalSyncPolarityCheck_CheckedChanged(object sender, EventArgs e)
        {
            if ((Platform == Platforms.CM_T35) && !bIsCustom)
            {
                this.PreDefined.SelectedItem = Predefines.Custom;
                bIsCustom = true;
            }
            UpdateSettingsForExport();
        }

        private void VertBackPorchEdit_TextChanged(object sender, EventArgs e)
        {
            if ((Platform == Platforms.CM_T35) && !bIsCustom)
            {
                this.PreDefined.SelectedItem = Predefines.Custom;
                bIsCustom = true;
            }
            UpdateSettingsForExport();
        }

        private void VertFrontPorchEdit_TextChanged(object sender, EventArgs e)
        {
            if ((Platform == Platforms.CM_T35) && !bIsCustom)
            {
                this.PreDefined.SelectedItem = Predefines.Custom;
                bIsCustom = true;
            }
            UpdateSettingsForExport();
        }

        private void VerticalSyncPolarityCheck_CheckedChanged(object sender, EventArgs e)
        {
            if ((Platform == Platforms.CM_T35) && !bIsCustom)
            {
                this.PreDefined.SelectedItem = Predefines.Custom;
                bIsCustom = true;
            }
            UpdateSettingsForExport();
        }

        private void ClockPolarityCheck_CheckedChanged(object sender, EventArgs e)
        {
            if ((Platform == Platforms.CM_T35) && !bIsCustom)
            {
                this.PreDefined.SelectedItem = Predefines.Custom;
                bIsCustom = true;
            }
            UpdateSettingsForExport();
        }
        #endregion  

    }
}